<script type="text/ng-template" id="cat_renderer.html">
  <p ng-class="{'cat-tree-category':cat.type=='menuCat'}">{{cat.title}}</p>
  <ul class="cat-tree"><li ng-repeat="cat in cat.nodes track by cat.id" ng-include="'cat_renderer.html'"></li></ul>
</script>
<div class="panel panel-default">
  <div class="panel-heading">
    <h3 class="panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
  <div class="panel-body">
  <ul class="cat-tree">
  <li ng-repeat="cat in currentNode.nodes track by cat.id" ng-include="'cat_renderer.html'"></li>
  </ul>
  </div>
</div>
<script type="text/menu">
[{
  "title": "newCat",
  "icon": "plus-sign"
}]
</script>
