<div class="panel panel-default llcontainer">
  <div class="panel-heading">
    <h3 class="panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
  <div class="panel-body">
    <div class="input-group maxw">
      <label class="input-group-addon" for="longtextinput" trspan="value"></label>
      <textarea id="longtextinput" rows="5" class="form-control" ng-model="currentNode.data"/>
    </div>
  </div>
</div>
<script type="text/menu">
[{
  "title": "cancel",
  "icon": "remove"
}]
</script>
