<div class="panel panel-default">
  <div class="panel-heading">
    <h3 class=panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
  <div class="panel-body">
    <div class="input-group-solid" role="radiogroup">
      <label class="radio-inline">
        <input id="blacklist" type="radio" value="0" name="bwl" ng-model="currentNode.data[0]" aria-labelledby="label0" role="radio" />
        <label id="label0" for="blacklist" trspan="blackList"></label>
      </label>
      <label class="radio-inline">
        <input id="whitelist" type="radio" value="1" name="bwl" ng-model="currentNode.data[0]" aria-labelledby="label1" role="radio" />
        <span id="label1" for="whitelist" trspan="whiteList"></span>
      </label>
    </div>
  </div>
  <div class="panel-body input-group">
    <label id="bwlabel" class="input-group-addon" for="bwlist" trspan="value"></label>
    <input id="bwlist" class="form-control" ng-model="currentNode.data[1]" aria-describedby="bwlabel"/>
  </div>
</div>
<script type="text/menu">
[]
</script>
